@@ -1,22 +1,3 @@
-array2list <- function(x) {
-  # convert array to list of elements with reduced dimension
-  # Args: 
-  #   x: an arrary of dimension d
-  # Returns: 
-  #   A list of arrays of dimension d-1
-  if (is.null(dim(x))) {
-    stop("Argument 'x' has no dimension.")
-  }
-  ndim <- length(dim(x))
-  l <- list(length = dim(x)[ndim])
-  ind <- collapse(rep(",", ndim - 1))
-  for (i in seq_len(dim(x)[ndim])) {
-    l[[i]] <- eval(parse(text = paste0("x[", ind, i, "]"))) 
-  }
-  names(l) <- dimnames(x)[[ndim]]
-  l
-}
-
 contains_samples <- function(x) {
   if (!(is.brmsfit(x) && length(x$fit@sim))) {
     stop2("The model does not contain posterior samples.")
@@ -30,6 +11,14 @@ algorithm <- function(x) {
   else x$algorithm
 }
 
+get_all_group_vars <- function(x) {
+  # extract names of all grouping variables
+  if (is.brmsfit(x)) {
+    x <- x$ranef
+  }
+  unique(ulapply(x$gcall, "[[", "groups"))
+}
+
 name_model <- function(family) {
   # create the name of the fitted stan model
   # Args:
@@ -81,11 +70,16 @@ restructure <- function(x, rstr_summary = FALSE) {
     # slot 'nonlinear' deprecated as of brms > 0.9.1
     x$formula <- SW(amend_formula(
       formula(x), data = model.frame(x), 
-      family = family(x), nonlinear = x$nonlinear
+      family = family(x), autocor = x$autocor,
+      nonlinear = x$nonlinear
     ))
     x$nonlinear <- x$partial <- NULL
     x$formula[["old_mv"]] <- is_old_mv(x)
-    bterms <- parse_bf(formula(x), family = family(x))
+    bterms <- parse_bf(formula(x))
+    if (!isTRUE(x$formula[["old_mv"]])) {
+      x$data <- rm_attr(x$data, "brmsframe")
+      x$data <- update_data(x$data, bterms) 
+    }
     x$ranef <- tidy_ranef(bterms, model.frame(x))
     if ("prior_frame" %in% class(x$prior)) {
       class(x$prior) <- c("brmsprior", "data.frame") 
@@ -140,6 +134,12 @@ restructure <- function(x, rstr_summary = FALSE) {
     if (version <= "1.8.0.1") {
       x$prior[, c("resp", "dpar")] <- ""
     }
+    if (version <= "1.9.0.3") {
+      # names of monotonic parameters had to be changed after
+      # allowing for interactions in monotonic terms
+      change <- change_old_mo(bterms, x$data, pars = parnames(x))
+      x <- do_renaming(x, change)
+    }
     stan_env <- attributes(x$fit)$.MISC
     if (rstr_summary && exists("summary", stan_env)) {
       stan_summary <- get("summary", stan_env)
@@ -154,18 +154,6 @@ restructure <- function(x, rstr_summary = FALSE) {
   structure(x, restructured = TRUE)
 }
 
-first_greater <- function(A, target, i = 1) {
-  # find the first element in A that is greater than target
-  # Args: 
-  #   A: a matrix
-  #   target: a vector of length nrow(A)
-  #   i: column of A being checked first
-  # Returns: 
-  #   A vector of the same length as target containing the column ids 
-  #   where A[,i] was first greater than target
-  ifelse(target <= A[, i] | ncol(A) == i, i, first_greater(A, target, i + 1))
-}
-
 link <- function(x, link) {
   # apply a link function on x
   # Args:
@@ -216,176 +204,6 @@ ilink <- function(x, link) {
   )
 }
 
-prepare_conditions <- function(x, conditions = NULL, effects = NULL,
-                               re_formula = NA, rsv_vars = NULL) {
-  # prepare marginal conditions
-  # Args:
-  #   x: an object of class 'brmsfit'
-  #   conditions: optional data.frame containing user defined conditions
-  #   effects: see marginal_effects
-  #   re_formula: see marginal_effects
-  #   rsv_vars: names of reserved variables
-  # Returns:
-  #   A data.frame with (possibly updated) conditions
-  mf <- model.frame(x)
-  new_formula <- update_re_terms(formula(x), re_formula = re_formula)
-  bterms <- parse_bf(new_formula, family = family(x))
-  re <- get_re(bterms)
-  req_vars <- c(
-    lapply(get_effect(bterms, "fe"), rhs), 
-    lapply(get_effect(bterms, "mo"), rhs),
-    lapply(get_effect(bterms, "me"), rhs),
-    lapply(get_effect(bterms, "sm"), rhs),
-    lapply(get_effect(bterms, "cs"), rhs),
-    lapply(get_effect(bterms, "gp"), rhs),
-    lapply(get_effect(bterms, "offset"), rhs),
-    re$form, lapply(re$gcall, "[[", "weightvars"),
-    bterms$adforms[c("se", "disp", "trials", "cat")],
-    bterms$dpars$mu$covars
-  )
-  req_vars <- unique(ulapply(req_vars, all.vars))
-  req_vars <- setdiff(req_vars, rsv_vars)
-  if (is.null(conditions)) {
-    conditions <- as.data.frame(as.list(rep(NA, length(req_vars))))
-    names(conditions) <- req_vars
-  } else {
-    conditions <- as.data.frame(conditions)
-    if (!nrow(conditions)) {
-      stop2("Argument 'conditions' must have a least one row.")
-    }
-    if (any(duplicated(rownames(conditions)))) {
-      stop2("Row names of 'conditions' should be unique.")
-    }
-    conditions <- unique(conditions)
-    req_vars <- setdiff(req_vars, names(conditions))
-  }
-  trial_vars <- all.vars(bterms$adforms$trials)
-  if (length(trial_vars)) {
-    write_msg <- any(ulapply(trial_vars, function(x) 
-      !isTRUE(x %in% names(conditions)) || anyNA(conditions[[x]])
-    ))
-    if (write_msg) {
-      message("Using the median number of trials by ", 
-              "default if not specified otherwise.")
-    }
-  }
-  # use default values for unspecified variables
-  int_vars <- rmNULL(bterms$adforms[c("trials", "cat")])
-  int_vars <- c(int_vars, get_effect(bterms, "mo"))
-  int_vars <- unique(ulapply(int_vars, all.vars))
-  for (v in req_vars) {
-    if (!is_like_factor(mf[[v]])) {
-      # treat variable as numeric
-      if (v %in% int_vars) {
-        conditions[[v]] <- round(median(mf[[v]]))
-      } else {
-        conditions[[v]] <- mean(mf[[v]])
-      }
-    } else {
-      # use reference category for factors
-      levels <- attr(as.factor(mf[[v]]), "levels")
-      conditions[[v]] <- factor(
-        levels[1], levels = levels, ordered = is.ordered(mf[[v]])
-      )
-    }
-  }
-  unused_vars <- setdiff(names(conditions), all.vars(bterms$allvars))
-  if (length(unused_vars)) {
-    warning2(
-      "The following variables in 'conditions' are not ", 
-      "part of the model:\n", collapse_comma(unused_vars)
-    )
-  }
-  amend_newdata(
-    conditions, fit = x, re_formula = re_formula,
-    allow_new_levels = TRUE, incl_autocor = FALSE, 
-    return_standata = FALSE
-  )
-}
-
-prepare_marg_data <- function(data, conditions, int_conditions = NULL,
-                              int_vars = NULL, surface = FALSE, 
-                              resolution = 100, reorder = TRUE) {
-  # prepare data to be used in marginal_effects
-  # Args:
-  #  data: data.frame containing only data of the predictors of interest
-  #  conditions: see argument 'conditions' of marginal_effects
-  #  int_conditions: see argument 'int_conditions' of marginal_effects
-  #  int_vars: names of variables being treated as integers
-  #  surface: generate surface plots later on?
-  #  resolution: number of distinct points at which to evaluate
-  #              the predictors of interest
-  #  reorder: reorder predictors so that numeric ones come first?
-  effects <- names(data)
-  stopifnot(length(effects) %in% c(1L, 2L))
-  pred_types <- ifelse(ulapply(data, is_like_factor), "factor", "numeric")
-  # numeric effects should come first
-  if (reorder) {
-    new_order <- order(pred_types, decreasing = TRUE)
-    effects <- effects[new_order]
-    pred_types <- pred_types[new_order]
-  }
-  mono <- effects %in% int_vars
-  if (pred_types[1] == "numeric") {
-    min1 <- min(data[, effects[1]])
-    max1 <- max(data[, effects[1]])
-    if (mono[1]) {
-      values <- seq(min1, max1, by = 1)
-    } else {
-      values <- seq(min1, max1, length.out = resolution)
-    }
-  } else {
-    values <- unique(data[, effects[1]])
-  }
-  if (length(effects) == 2L) {
-    values <- setNames(list(values, NA), effects)
-    if (pred_types[2] == "numeric") {
-      if (surface) {
-        min2 <- min(data[, effects[2]])
-        max2 <- max(data[, effects[2]])
-        if (mono[2]) {
-          values[[2]] <- seq(min2, max2, by = 1)
-        } else {
-          values[[2]] <- seq(min2, max2, length.out = resolution)
-        }
-      } else {
-        if (effects[2] %in% names(int_conditions)) {
-          int_cond <- int_conditions[[effects[2]]]
-          if (is.function(int_cond)) {
-            int_cond <- int_cond(data[, effects[2]])
-          }
-          values[[2]] <- int_cond
-        } else if (mono[2]) {
-          median2 <- median(data[, effects[2]])
-          mad2 <- mad(data[, effects[2]])
-          values[[2]] <- round((-1:1) * mad2 + median2)
-        } else {
-          mean2 <- mean(data[, effects[2]])
-          sd2 <- sd(data[, effects[2]])
-          values[[2]] <- (-1:1) * sd2 + mean2
-        }
-      }
-    } else {
-      values[[2]] <- unique(data[, effects[2]])
-    }
-    data <- do.call(expand.grid, values)
-  } else {
-    data <- structure(data.frame(values), names = effects)
-  }
-  # no need to have the same value combination more than once
-  data <- unique(data)
-  data <- data[do.call(order, as.list(data)), , drop = FALSE]
-  data <- replicate(nrow(conditions), data, simplify = FALSE)
-  marg_vars <- setdiff(names(conditions), effects)
-  for (j in seq_len(nrow(conditions))) {
-    data[[j]][, marg_vars] <- conditions[j, marg_vars]
-    data[[j]]$cond__ <- rownames(conditions)[j]
-  }
-  data <- do.call(rbind, data)
-  data$cond__ <- factor(data$cond__, rownames(conditions))
-  structure(data, effects = effects, types = pred_types, mono = mono)
-}
-
 subset_samples <- function(x, subset = NULL, nsamples = NULL) {
   # generate integers indicating subsets of the posterior samples
   stopifnot(is.brmsfit(x))
@@ -827,7 +645,7 @@ choose_N <- function(draws) {
 prepare_family <- function(x) {
   # prepare for calling family specific log_lik / predict functions
   family <- family(x)
-  nresp <- length(parse_bf(x$formula, family = family)$response)
+  nresp <- length(parse_bf(x$formula)$response)
   if (is_old_lognormal(family, nresp = nresp, version = x$version$brms)) {
     family <- lognormal()
   } else if (is_linear(family) && nresp > 1L) {
@@ -931,12 +749,12 @@ find_names <- function(x) {
   if (!is.character(x) || length(x) > 1) {
     stop2("Argument 'x' must be a character string of length one.")
   }
-  x <- gsub(" ", "", x)
-  reg_all <- paste0("([^([:digit:]|[:punct:])]|\\.)[[:alnum:]_\\.\\:]*", 
-                    "(\\[[^],]+(,[^],]+)*\\])?")
-  pos_all <- gregexpr(reg_all, x)[[1]]
-  reg_fun <- "([^([:digit:]|[:punct:])]|\\.)[[:alnum:]_\\.]*\\("
-  pos_fun <- gregexpr(reg_fun, x)[[1]]
+  x <- gsub("[[:space:]]", "", x)
+  regex_all <- "([^([:digit:]|[:punct:])]|\\.)[[:alnum:]_\\.\\:]*"
+  regex_all <- paste0(regex_all, "(\\[[^],]+(,[^],]+)*\\])?")
+  pos_all <- gregexpr(regex_all, x)[[1]]
+  regex_fun <- "([^([:digit:]|[:punct:])]|\\.)[[:alnum:]_\\.]*\\("
+  pos_fun <- gregexpr(regex_fun, x)[[1]]
   pos_decnum <- gregexpr("\\.[[:digit:]]+", x)[[1]]
   pos_var <- list(rmMatch(pos_all, pos_fun, pos_decnum))
   unlist(regmatches(x, pos_var))
@@ -986,90 +804,6 @@ evidence_ratio <- function(x, cut = 0, wsign = c("equal", "less", "greater"),
   out
 }
 
-make_point_frame <- function(mf, effects, conditions, groups, 
-                             family, select_points = 0, 
-                             transform = NULL) {
-  # helper function for marginal_effects.brmsfit
-  # allowing add data points to the marginal plots
-  # Args:
-  #   mf: the original model.frame
-  #   effects: see marginal_effects
-  #   conditions: see marginal_effects
-  #   groups: names of the grouping factors
-  #   family: the model family
-  #   select_points: see marginal_effects
-  # Returns:
-  #   a data.frame containing the data points to be plotted
-  points <- mf[, effects, drop = FALSE]
-  points$resp__ <- model.response(mf)
-  req_vars <- names(mf)
-  if (length(groups)) {
-    req_vars <- c(req_vars, unlist(strsplit(groups, ":")))
-  }
-  req_vars <- unique(setdiff(req_vars, effects))
-  req_vars <- intersect(req_vars, names(conditions))
-  if (length(req_vars)) {
-    # find out which data point is valid for which condition
-    mf <- mf[, req_vars, drop = FALSE]
-    conditions <- conditions[, req_vars, drop = FALSE]
-    points$cond__ <- NA
-    points <- replicate(nrow(conditions), points, simplify = FALSE)
-    for (i in seq_along(points)) {
-      cond <- conditions[i, , drop = FALSE]
-      not_na <- !(is.na(cond) | cond == "zero__")
-      cond <- cond[, not_na, drop = FALSE]
-      mf_tmp <- mf[, not_na, drop = FALSE]
-      if (ncol(mf_tmp)) {
-        is_num <- sapply(mf_tmp, is.numeric) 
-        is_num <- is_num & !names(mf_tmp) %in% groups
-        if (sum(is_num)) {
-          # handle numeric variables
-          stopifnot(select_points >= 0)
-          if (select_points > 0) {
-            for (v in names(mf_tmp)[is_num]) {
-              min <- min(mf_tmp[, v])
-              max <- max(mf_tmp[, v])
-              unit <- scale_unit(mf_tmp[, v], min, max)
-              unit_cond <- scale_unit(cond[, v], min, max)
-              unit_diff <- abs(unit - unit_cond)
-              close_enough <- unit_diff <= select_points
-              mf_tmp[close_enough, v] <- cond[, v]
-              mf_tmp[!close_enough, v] <- NA
-            }
-          } else {
-            # take all numeric values if select_points is zero
-            cond <- cond[, !is_num, drop = FALSE]
-            mf_tmp <- mf_tmp[, !is_num, drop = FALSE]
-          }
-        }
-      }
-      if (ncol(mf_tmp)) {
-        # handle factors and grouping variables
-        # do it like base::duplicated
-        K <- do.call("paste", c(mf_tmp, sep = "\r")) %in%
-             do.call("paste", c(cond, sep = "\r"))
-      } else {
-        K <- seq_len(nrow(mf))
-      }
-      # cond__ allows to assign points to conditions
-      points[[i]]$cond__[K] <- rownames(conditions)[i]
-    }
-    points <- do.call(rbind, points)
-    points <- points[!is.na(points$cond__), , drop = FALSE]
-    points$cond__ <- factor(points$cond__, rownames(conditions))
-  }
-  if (!is.numeric(points$resp__)) {
-    points$resp__ <- as.numeric(as.factor(points$resp__))
-    if (is_binary(family)) {
-      points$resp__ <- points$resp__ - 1
-    }
-  }
-  if (!is.null(transform)) {
-    points$resp__ <- do.call(transform, list(points$resp__))
-  }
-  points
-}
-
 hypothesis_internal <- function(x, hypothesis, class = "", alpha = 0.05, ...) {
   # internal function to evaluate hypotheses
   # Args:
@@ -1158,14 +892,15 @@ hypothesis_internal <- function(x, hypothesis, class = "", alpha = 0.05, ...) {
   pars <- parnames(x)[grepl("^", class, parnames(x))]
   hlist <- lapply(hypothesis, .eval_hypothesis)
   hs <- do.call(rbind, lapply(hlist, function(h) h$summary))
-  samples <- do.call(cbind, lapply(hlist, function(h) h$samples))
-  samples <- cbind(as.data.frame(samples), Type = "Posterior")
-  prior_samples <- do.call(cbind, lapply(hlist, function(h) h$prior_samples))
-  prior_samples <- cbind(as.data.frame(prior_samples), Type = "Prior")
-  samples <- rbind(samples, prior_samples)
-  names(samples) <- c(paste0("H", seq_along(hlist)), "Type")
+  samples <- as.data.frame(
+    do.call(cbind, lapply(hlist, function(h) h$samples))
+  )
+  prior_samples <- as.data.frame(
+    do.call(cbind, lapply(hlist, function(h) h$prior_samples))
+  )
+  names(samples) <- names(prior_samples) <- paste0("H", seq_along(hlist))
   class <- sub("_+$", "", class)
-  out <- nlist(hypothesis = hs, samples, class, alpha)
+  out <- nlist(hypothesis = hs, samples, prior_samples, class, alpha)
   class(out) <- "brmshypothesis"
   out
 }
