@@ -1,3 +1,8 @@
+extract_draws <- function(x, ...) {
+  # extract data and posterior draws
+  UseMethod("extract_draws")
+}
+
 #' @export
 extract_draws.brmsfit <- function(x, newdata = NULL, re_formula = NULL, 
                                   allow_new_levels = FALSE, 
@@ -13,8 +18,9 @@ extract_draws.brmsfit <- function(x, newdata = NULL, re_formula = NULL,
   #   A named list to be interpreted by linear_predictor
   snl_options <- c("uncertainty", "gaussian", "old_levels")
   sample_new_levels <- match.arg(sample_new_levels, snl_options)
+  x <- restructure(x)
   x <- remove_autocor(x, incl_autocor)
-  bterms <- parse_bf(formula(x), family = family(x))
+  bterms <- parse_bf(x$formula)
   subset <- subset_samples(x, subset, nsamples)
   nsamples <- nsamples(x, subset = subset)
   newd_args <- nlist(
@@ -46,26 +52,27 @@ extract_draws.brmsfit <- function(x, newdata = NULL, re_formula = NULL,
     }
     bterms$dpars[["mu"]] <- NULL
   }
-  # extract draws of auxiliary parameters
+  # extract draws of distributional parameters
   am_args <- nlist(x, subset)
-  valid_dpars <- valid_dpars(family(x), bterms = bterms)
-  for (ap in valid_dpars) {
-    ap_regex <- paste0("^", ap, "($|_)")
-    if (is.btl(bterms$dpars[[ap]]) || is.btnl(bterms$dpars[[ap]])) {
-      p <- usc(combine_prefix(bterms$dpars[[ap]]))
-      more_args <- nlist(
-        x = bterms$dpars[[ap]], C = draws$data[[paste0("C", p)]]
-      )
-      draws[[ap]] <- do.call(extract_draws, c(args, more_args))
-      draws[[ap]][["f"]] <- bterms$dpars[[ap]]$family
-    } else if (is.numeric(bterms$fdpars[[ap]]$value)) {
-      draws[[ap]] <- bterms$fdpars[[ap]]$value
-    } else if (any(grepl(ap_regex, parnames(x)))) {
-      draws[[ap]] <- do.call(as.matrix, c(am_args, pars = ap_regex))
+  valid_dpars <- valid_dpars(x$family, bterms = bterms)
+  for (dp in valid_dpars) {
+    dp_regex <- paste0("^", dp, "($|_)")
+    if (is.btl(bterms$dpars[[dp]]) || is.btnl(bterms$dpars[[dp]])) {
+      more_args <- list(x = bterms$dpars[[dp]])
+      if (is.btnl(bterms$dpars[[dp]])) {
+        p <- usc(combine_prefix(bterms$dpars[[dp]]))
+        more_args$C <- draws$data[[paste0("C", p)]]
+      }
+      draws[[dp]] <- do.call(extract_draws, c(args, more_args))
+      draws[[dp]][["f"]] <- bterms$dpars[[dp]]$family
+    } else if (is.numeric(bterms$fdpars[[dp]]$value)) {
+      draws[[dp]] <- bterms$fdpars[[dp]]$value
+    } else if (any(grepl(dp_regex, parnames(x)))) {
+      draws[[dp]] <- do.call(as.matrix, c(am_args, pars = dp_regex))
     }
   }
-  if (is.mixfamily(family(x))) {
-    families <- family_names(family(x))
+  if (is.mixfamily(x$family)) {
+    families <- family_names(x$family)
     thetas <- paste0("theta", seq_along(families))
     if (any(ulapply(draws[thetas], is.list))) {
       # theta was predicted
@@ -85,7 +92,7 @@ extract_draws.brmsfit <- function(x, newdata = NULL, re_formula = NULL,
       }
     }
   }
-  if (is_linear(family(x)) && length(bterms$response) > 1L) {
+  if (is_linear(x$family) && length(bterms$response) > 1L) {
     # parameters for multivariate normal models
     draws$sigma <- do.call(as.matrix, c(am_args, pars = "^sigma($|_)"))
     draws$rescor <- do.call(as.matrix, c(am_args, pars = "^rescor_"))
@@ -94,11 +101,10 @@ extract_draws.brmsfit <- function(x, newdata = NULL, re_formula = NULL,
   if (use_cov(x$autocor) || is.cor_sar(x$autocor)) {
     # only include autocor samples on the top-level of draws 
     # when using the covariance formulation of ARMA / SAR structures
-    draws <- c(draws, 
-      extract_draws_autocor(
-        x, subset, newdata = newdata, sdata = draws$data
-      )
+    draws_autocor <- extract_draws_autocor(
+      x, subset, newdata = newdata, sdata = draws$data
     )
+    draws <- c(draws, draws_autocor)
   }
   draws
 }
@@ -140,80 +146,78 @@ extract_draws.btl <- function(x, fit, newdata = NULL, re_formula = NULL,
   #   other arguments: see extract_draws.brmsfit
   # Returns:
   #   A named list to be interpreted by linear_predictor
+  stopifnot(is.brmsfit(fit))
   dots <- list(...)
-  stopifnot(is.brmsfit(fit), is.brmsformula(fit$formula))
   fit <- remove_autocor(fit, incl_autocor)
   px <- check_prefix(x)
   p <- combine_prefix(px)
   nsamples <- nsamples(fit, subset = subset)
+  all_group_vars <- get_all_group_vars(fit)
   fit$formula$formula <- update(fit$formula$formula, rhs(x$formula))
   # ensure that auxiliary parameters are not included (fixes #154)
   fit$formula$pforms <- fit$formula$pfix <- NULL
-  fit$formula$nl <- FALSE
-  fit$ranef <- tidy_ranef(parse_bf(fit$formula), data = fit$data)
+  fit$ranef <- tidy_ranef(parse_bf(fit$formula), fit$data)
   if (nzchar(p)) {
     # make sure not to evaluate family specific stuff
     fit$formula[["response"]] <- NA
     fit$family <- fit$formula$family <- .dpar_family()
   }
-  new_formula <- update_re_terms(fit$formula, re_formula = re_formula)
-  bterms <- parse_bf(new_formula, family = family(fit))
-  new_ranef <- tidy_ranef(bterms, model.frame(fit))
+  new_formula <- update_re_terms(fit$formula, re_formula)
+  bterms <- parse_bf(new_formula)
+  new_ranef <- tidy_ranef(bterms, fit$data)
   newd_args <- nlist(
     fit, newdata, re_formula, allow_new_levels, 
-    new_objects, check_response = FALSE
-  )
-  draws <- list(
-    # supresses messages of add_new_objects
-    data = suppressMessages(do.call(amend_newdata, newd_args)), 
-    old_cat = is_old_categorical(fit)
+    new_objects, all_group_vars, check_response = FALSE
   )
+  # supresses messages of add_new_objects
+  sdata <- suppressMessages(do.call(amend_newdata, newd_args))
+  draws <- list(data = sdata, old_cat = is_old_categorical(fit))
   draws[names(dots)] <- dots
-  
-  if (smooths_only) {
-    # make sure only smooth terms will be included in draws
-    keep_elements <- setdiff(names(draws$data), c("Xm", "Xcs", "offset"))
-    draws$data <- draws$data[keep_elements]
-    new_ranef <- empty_ranef()
-  }
-  
   args <- nlist(x = fit, subset)
   new <- !is.null(newdata)
   # deviate from the usual way of passing bterms and data
   # as bterms and px do not coincide in extract_draws.btl
-  fixef <- colnames(draws$data[["X"]])
-  monef <- colnames(draws$data[["Xmo"]])
-  csef <- colnames(draws$data[["Xcs"]])
-  meef <- get_me_labels(bterms$dpars$mu, fit$data)
-  smooths <- get_sm_labels(bterms$dpars$mu, fit$data, covars = TRUE)
-  gpef <- get_gp_labels(bterms$dpars$mu, fit$data, covars = TRUE)
-  draws <- c(draws,
-    extract_draws_fe(fixef, args, px = px, old_cat = draws$old_cat),
-    extract_draws_mo(monef, args, sdata = draws$data, px = px),
-    extract_draws_cs(csef, args, px = px, old_cat = draws$old_cat),
-    extract_draws_me(meef, args, sdata = draws$data, px = px, new = new),
-    extract_draws_sm(smooths, args, sdata = draws$data, px = px),
-    extract_draws_gp(
-      gpef, args, sdata = draws$data, px = px, 
-      new = new, nug = nug, newd_args = newd_args
-    ),
-    extract_draws_re(
-      new_ranef, args, sdata = draws$data, px = px,
-      sample_new_levels = sample_new_levels
-    )
+  fixef <- colnames(sdata[["X"]])
+  draws$fe <- extract_draws_fe(
+    fixef, args, sdata, px = px, old_cat = draws$old_cat
   )
-  if (!use_cov(fit$autocor) && (!nzchar(p) || mv)) {
-    # only include autocorrelation parameters in draws for mu
-    draws <- c(draws, 
-      extract_draws_autocor(
-        fit, subset, newdata = newdata, sdata = draws$data
+  smooths <- get_sm_labels(bterms$dpars$mu, fit$data, covars = TRUE)
+  draws$sm <- extract_draws_sm(smooths, args, sdata, px = px)
+  if (smooths_only) {
+    # make sure only smooth terms will be included in draws
+    return(structure(draws, predicted = TRUE))
+  }
+  monef <- get_mo_labels(bterms$dpars$mu, fit$data)
+  draws$mo <- extract_draws_mo(monef, args, sdata, px = px)
+  csef <- colnames(sdata[["Xcs"]])
+  draws$cs <- extract_draws_cs(
+    csef, args, sdata, px = px, old_cat = draws$old_cat
+  )
+  meef <- get_me_labels(bterms$dpars$mu, fit$data)
+  draws$me <- extract_draws_me(meef, args, sdata, px = px, new = new)
+  gpef <- get_gp_labels(bterms$dpars$mu, fit$data, covars = TRUE)
+  draws$gp <- extract_draws_gp(
+    gpef, args, sdata, px = px, new = new, 
+    nug = nug, newd_args = newd_args
+  )
+  draws$re <- extract_draws_re(
+    new_ranef, args, sdata, px = px,
+    sample_new_levels = sample_new_levels
+  )
+  if (!nzchar(p) || mv) {
+    # only include autocorrelation parameters in draws of mu
+    if (!(use_cov(fit$autocor) || is.cor_sar(fit$autocor))) {
+      draws_autocor <- extract_draws_autocor(
+        fit, subset, newdata = newdata, sdata = sdata
       )
-    )
+      draws <- c(draws, draws_autocor)
+    }
   }
   structure(draws, predicted = TRUE)
 }
 
-extract_draws_fe <- function(fixef, args, px = list(), old_cat = 0L) {
+extract_draws_fe <- function(fixef, args, sdata, 
+                             px = list(), old_cat = 0L) {
   # extract draws of ordinary population-level effects
   # Args:
   #   fixef: names of the population-level effects
@@ -223,13 +227,14 @@ extract_draws_fe <- function(fixef, args, px = list(), old_cat = 0L) {
   # Returns: 
   #   A named list to be interpreted by linear_predictor
   stopifnot("x" %in% names(args))
-  p <- usc(combine_prefix(px), "suffix")
   draws <- list()
   if (length(fixef) && old_cat != 1L) {
+    p <- usc(combine_prefix(px), "suffix")
     b_pars <- paste0("b_", p, fixef)
-    draws[["b"]] <- do.call(as.matrix, 
+    draws$b <- do.call(as.matrix, 
       c(args, list(pars = b_pars, exact = TRUE))
     )
+    draws$X <- sdata[["X"]]
   }
   draws
 }
@@ -240,35 +245,55 @@ extract_draws_mo <- function(monef, args, sdata, px = list()) {
   #   monef: names of the monotonic effects
   #   args: list of arguments passed to as.matrix.brmsfit
   #   sdata: list returned by make_standata
-  #   nlpar: name of a non-linear parameter
   # Returns: 
   #   A named list to be interpreted by linear_predictor
   stopifnot("x" %in% names(args))
-  p <- usc(combine_prefix(px), "suffix")
   draws <- list()
-  if (length(monef)) {
-    draws[["bmo"]] <- draws$simplex <- named_list(monef)
-    # as of brms > 1.2.0 the prefix 'bmo' is used
-    bmo <- ifelse(
-      any(grepl("^bmo_", parnames(args$x))), "bmo_",
-      ifelse(any(grepl("^bm_", parnames(args$x))), "bm_", "b_")
-    )
-    for (i in seq_along(monef)) {
-      bmo_par <- paste0(bmo, p, monef[i])
-      draws[["bmo"]][[i]] <- do.call(as.matrix,
-        c(args, list(pars = bmo_par, exact = TRUE))
-      )
-      simplex_par <- paste0("simplex_", p, monef[i], 
-                            "[", seq_len(sdata$Jm[i]), "]")
-      draws[["simplex"]][[i]] <- do.call(as.matrix, 
-        c(args, list(pars = simplex_par, exact = TRUE))
-      )
+  if (!length(monef)) {
+    return(draws) 
+  }
+  p <- usc(combine_prefix(px), "suffix")
+  att <- attributes(monef)
+  # prepare calls evaluated in 'mo_predictor()'
+  calls <- rep(NA, length(monef))
+  for (i in seq_along(calls)) {
+    tmp <- paste0(".mo(simo_", att$Imo[[i]], ", Xmo_", att$Imo[[i]], ")")
+    calls[i] <- paste0(tmp, collapse = " * ")
+    if (att$not_one[i]) {
+      str_add(calls[i]) <- paste0(" * Cmo_", att$Icmo[i])
     }
   }
+  draws$calls <- parse(text = paste0(calls))
+  # extract data and parameters for monotonic effects
+  monef <- rename(monef)
+  draws$bmo <- named_list(monef)
+  for (i in seq_along(monef)) {
+    bmo_par <- paste0("bmo_", p, monef[i])
+    draws$bmo[[i]] <- do.call(as.matrix,
+      c(args, list(pars = bmo_par, exact = TRUE))
+    )
+  }
+  simo_coef <- get_simo_labels(monef)
+  draws$simo <- draws$Xmo <- named_list(simo_coef)
+  for (i in seq_along(simo_coef)) {
+    simo_par <- paste0(
+      "simo_", p, simo_coef[i], "[", seq_len(sdata$Jmo[i]), "]"
+    )
+    draws$simo[[i]] <- do.call(as.matrix, 
+      c(args, list(pars = simo_par, exact = TRUE))
+    )
+    draws$Xmo[[i]] <- sdata[[paste0("Xmo_", i)]]
+  }
+  ncovars <- sum(att$not_one)
+  dim <- c(nrow(draws$bmo[[1]]), sdata[["N"]])
+  for (i in seq_len(ncovars)) {
+    draws$Cmo[[i]] <- as_draws_matrix(sdata[[paste0("Cmo_", i)]], dim)
+  }
   draws
 }
 
-extract_draws_cs <- function(csef, args, px = list(), old_cat = 0L) {
+extract_draws_cs <- function(csef, args, sdata, 
+                             px = list(), old_cat = 0L) {
   # category specific effects 
   # extract draws of category specific effects
   # Args:
@@ -281,7 +306,7 @@ extract_draws_cs <- function(csef, args, px = list(), old_cat = 0L) {
   stopifnot("x" %in% names(args))
   p <- usc(combine_prefix(px), "suffix")
   draws <- list()
-  if (is_ordinal(family(args$x))) {
+  if (is_ordinal(args$x$family)) {
     draws[["Intercept"]] <- do.call(as.matrix, 
       c(args, list(pars = "^b_Intercept\\["))
     )
@@ -289,11 +314,13 @@ extract_draws_cs <- function(csef, args, px = list(), old_cat = 0L) {
       # as of brms > 1.0.1 the original prefix 'bcs' is used
       bcs <- ifelse(any(grepl("^bcs_", parnames(args$x))), "^bcs_", "^b_")
       cs_pars <- paste0(bcs, p, csef, "\\[")
-      draws[["cs"]] <- do.call(as.matrix, c(args, list(pars = cs_pars)))
+      draws[["bcs"]] <- do.call(as.matrix, c(args, list(pars = cs_pars)))
+      draws[["Xcs"]] <- sdata[["Xcs"]]
     }
   } else if (old_cat == 1L) {
     # old categorical models deprecated as of brms > 0.8.0
-    draws[["cs"]] <- do.call(as.matrix, c(args, list(pars = "^b_")))
+    draws[["bcs"]] <- do.call(as.matrix, c(args, list(pars = "^b_")))
+    draws[["Xcs"]] <- sdata[["X"]]
   }
   draws
 }
@@ -310,63 +337,55 @@ extract_draws_me <- function(meef, args, sdata, px = list(),
   # Returns: 
   #   A named list to be interpreted by linear_predictor
   stopifnot("x" %in% names(args))
-  p <- usc(combine_prefix(px), "suffix")
   draws <- list()
-  if (length(meef)) {
-    if (new) {
-      stop2("Predictions with noise-free variables are not yet ",
-            "possible when passing new data.")
-    }
-    if (!any(grepl(paste0("Xme_", p), parnames(args$x)))) {
-      stop2("Noise-free variables were not saved. Please set ",
-            "argument 'save_mevars' to TRUE when calling 'brm'.")
-    }
-    uni_me <- attr(meef, "uni_me")
-    not_one <- attr(meef, "not_one")
-    # prepare calls to evaluate noise-free data
-    me_sp <- strsplit(gsub("[[:space:]]", "", meef), ":")
-    meef_terms <- rep(NA, length(me_sp))
-    for (i in seq_along(me_sp)) {
-      # remove non-me parts from the terms
-      take <- grepl_expr("^me\\([^:]*\\)$", me_sp[[i]])
-      me_sp[[i]] <- me_sp[[i]][take]
-      meef_terms[i] <- paste0(me_sp[[i]], collapse = " * ")
-    }
-    new_me <- paste0("Xme_", seq_along(uni_me))
-    meef_terms <- rename(meef_terms, uni_me, new_me)
-    ci <- ulapply(seq_along(not_one), function(i) sum(not_one[1:i]))
-    covars <- ifelse(not_one, paste0(" * Cme_", ci), "")
-    meef_terms <- paste0(meef_terms, covars)
-    # extract coefficient samples
-    meef <- rename(meef)
-    draws[["bme"]] <- named_list(meef)
-    attr(draws[["bme"]], "calls") <- parse(text = meef_terms)
-    for (j in seq_along(draws[["bme"]])) {
-      bme_par <- paste0("bme_", p, meef[j])
-      draws[["bme"]][[j]] <- do.call(as.matrix, 
-        c(args, list(pars = bme_par, exact = TRUE))
-      )
-    }
-    # extract noise-free variable samples
-    uni_me <- rename(uni_me)
-    draws[["Xme"]] <- named_list(uni_me)
-    for (j in seq_along(draws[["Xme"]])) {
-      Xme_pars <- paste0(
-        "Xme_", p, uni_me[j], "[", seq_len(nobs(args$x)), "]"
-      )
-      draws[["Xme"]][[j]] <- do.call(as.matrix,
-        c(args, list(pars = Xme_pars, exact = TRUE))
-      )
-    }
-    # prepare covariates
-    ncovars <- sum(not_one)
-    for (j in seq_len(ncovars)) {
-      cme <- paste0("Cme_", j)
-      draws[["Cme"]][[j]] <- as_draws_matrix(
-        sdata[[cme]], dim = dim(draws[["Xme"]][[1]])
-      )
-    }
-  } 
+  if (!length(meef)) {
+    return(draws) 
+  }
+  p <- usc(combine_prefix(px), "suffix")
+  if (new) {
+    stop2("Predictions with noise-free variables are not yet ",
+          "possible when passing new data.")
+  }
+  if (!any(grepl("^Xme_", parnames(args$x)))) {
+    stop2("Noise-free variables were not saved. Please set ",
+          "argument 'save_mevars' to TRUE when fitting your model.")
+  }
+  att <- attributes(meef)
+  uni_me <- att$uni_me
+  # prepare calls to evaluate noise-free data
+  meef_terms <- ulapply(att$calls_me, paste0, collapse = " * ")
+  new_me <- paste0("Xme_", seq_along(uni_me))
+  meef_terms <- rename(meef_terms, uni_me, new_me)
+  covars <- ifelse(att$not_one, paste0(" * Cme_", att$Icme), "")
+  meef_terms <- paste0(meef_terms, covars)
+  draws[["calls"]] <- parse(text = meef_terms)
+  # extract coefficient samples
+  meef <- rename(meef)
+  draws[["bme"]] <- named_list(meef)
+  for (j in seq_along(draws[["bme"]])) {
+    bme_par <- paste0("bme_", p, meef[j])
+    draws[["bme"]][[j]] <- do.call(as.matrix, 
+      c(args, list(pars = bme_par, exact = TRUE))
+    )
+  }
+  # extract noise-free variable samples
+  uni_me <- rename(uni_me)
+  draws[["Xme"]] <- named_list(uni_me)
+  N <- seq_len(nobs(args$x))
+  for (j in seq_along(draws[["Xme"]])) {
+    Xme_pars <- paste0("Xme_", uni_me[j], "[", N, "]")
+    draws[["Xme"]][[j]] <- do.call(as.matrix,
+      c(args, list(pars = Xme_pars, exact = TRUE))
+    )
+  }
+  # prepare covariates
+  ncovars <- sum(att$not_one)
+  for (j in seq_len(ncovars)) {
+    cme <- paste0("Cme_", j)
+    draws[["Cme"]][[j]] <- as_draws_matrix(
+      sdata[[cme]], dim = dim(draws[["Xme"]][[1]])
+    )
+  }
   draws
 }
 
@@ -380,19 +399,20 @@ extract_draws_sm <- function(smooths, args, sdata, px = list()) {
   # Returns: 
   #   A named list to be interpreted by linear_predictor
   stopifnot("x" %in% names(args))
+  if (!length(smooths)) {
+    return(list())
+  }
   p <- usc(combine_prefix(px), "suffix")
-  draws <- list()
-  if (length(smooths)) {
-    draws[["Zs"]] <- draws[["s"]] <- named_list(smooths)
-    for (i in seq_along(smooths)) {
-      nb <- seq_len(attr(smooths, "nbases")[[i]])
-      for (j in nb) {
-        draws[["Zs"]][[smooths[i]]][[j]] <- sdata[[paste0("Zs_", i, "_", j)]]
-        s_pars <- paste0("^s_", p, smooths[i], "_", j, "\\[")
-        draws[["s"]][[smooths[i]]][[j]] <- 
-          do.call(as.matrix, c(args, list(pars = s_pars)))
-      }
+  draws <- named_list(smooths)
+  for (i in seq_along(smooths)) {
+    sm <- list()
+    nb <- seq_len(attr(smooths, "nbases")[[i]])
+    for (j in nb) {
+      sm$Zs[[j]] <- sdata[[paste0("Zs_", i, "_", j)]]
+      s_pars <- paste0("^s_", p, smooths[i], "_", j, "\\[")
+      sm$s[[j]] <- do.call(as.matrix, c(args, list(pars = s_pars)))
     }
+    draws[[i]] <- sm
   }
   draws
 }
@@ -404,48 +424,48 @@ extract_draws_gp <- function(gpef, args, sdata, px = list(),
   # Args:
   #   gpef: names of the gaussian process terms
   stopifnot("x" %in% names(args))
+  if (!length(gpef)) {
+    return(list()) 
+  }
   sdata_old <- NULL
   if (length(gpef) && new) {
     newd_args["newdata"] <- list(NULL)
     sdata_old <- do.call(amend_newdata, newd_args)
   }
   p <- usc(combine_prefix(px), "suffix")
-  draws <- list()
-  if (length(gpef)) {
-    draws[["gp"]] <- named_list(gpef)
-    if (is.null(nug)) {
-      nug <- ifelse(new, 1e-8, 1e-11)
-    }
-    for (i in seq_along(gpef)) {
-      gp <- list()
-      by_levels <- attr(gpef, "by_levels")[[i]]
-      gp_names <- paste0(gpef[i], usc(by_levels))
-      sdgp <- paste0("^sdgp_", p, gp_names)
-      gp[["sdgp"]] <- do.call(as.matrix, c(args, list(pars = sdgp)))
-      lscale <- paste0("^lscale_", p, gp_names)
-      gp[["lscale"]] <- do.call(as.matrix, c(args, list(pars = lscale)))
-      zgp <- paste0("^zgp_", p, gpef[i], "\\[")
-      gp[["zgp"]] <- do.call(as.matrix, c(args, list(pars = zgp)))
-      gp[["bynum"]] <- sdata[[paste0("Cgp_", i)]]
-      Jgp_pos <- grepl(paste0("^Jgp_", i), names(sdata))
-      if (new) {
-        gp[["x"]] <- sdata_old[[paste0("Xgp_", i)]]
-        gp[["x_new"]] <- sdata[[paste0("Xgp_", i)]]
-        gp[["Jgp"]] <- sdata_old[Jgp_pos]
-        gp[["Jgp_new"]] <- sdata[Jgp_pos]
-        # computing GPs for new data requires the old GP terms
-        gp[["yL"]] <- gp_predictor(
-          x = gp[["x"]], sdgp = gp[["sdgp"]],
-          lscale = gp[["lscale"]], zgp = gp[["zgp"]], 
-          Jgp = gp[["Jgp"]]
-        )
-      } else {
-        gp[["x"]] <- sdata[[paste0("Xgp_", i)]]
-        gp[["Jgp"]] <- sdata[Jgp_pos]
-      }
-      gp[["nug"]] <- nug
-      draws[["gp"]][[i]] <- gp
+  if (is.null(nug)) {
+    nug <- ifelse(new, 1e-8, 1e-11)
+  }
+  draws <- named_list(gpef)
+  for (i in seq_along(gpef)) {
+    gp <- list()
+    by_levels <- attr(gpef, "by_levels")[[i]]
+    gp_names <- paste0(gpef[i], usc(by_levels))
+    sdgp <- paste0("^sdgp_", p, gp_names)
+    gp[["sdgp"]] <- do.call(as.matrix, c(args, list(pars = sdgp)))
+    lscale <- paste0("^lscale_", p, gp_names)
+    gp[["lscale"]] <- do.call(as.matrix, c(args, list(pars = lscale)))
+    zgp <- paste0("^zgp_", p, gpef[i], "\\[")
+    gp[["zgp"]] <- do.call(as.matrix, c(args, list(pars = zgp)))
+    gp[["bynum"]] <- sdata[[paste0("Cgp_", i)]]
+    Jgp_pos <- grepl(paste0("^Jgp_", i), names(sdata))
+    if (new) {
+      gp[["x"]] <- sdata_old[[paste0("Xgp_", i)]]
+      gp[["x_new"]] <- sdata[[paste0("Xgp_", i)]]
+      gp[["Jgp"]] <- sdata_old[Jgp_pos]
+      gp[["Jgp_new"]] <- sdata[Jgp_pos]
+      # computing GPs for new data requires the old GP terms
+      gp[["yL"]] <- .predictor_gp(
+        x = gp[["x"]], sdgp = gp[["sdgp"]],
+        lscale = gp[["lscale"]], zgp = gp[["zgp"]], 
+        Jgp = gp[["Jgp"]]
+      )
+    } else {
+      gp[["x"]] <- sdata[[paste0("Xgp_", i)]]
+      gp[["Jgp"]] <- sdata[Jgp_pos]
     }
+    gp[["nug"]] <- nug
+    draws[[i]] <- gp
   }
   draws
 }
@@ -462,8 +482,11 @@ extract_draws_re <- function(ranef, args, sdata, px = list(),
   # Returns: 
   #   A named list to be interpreted by linear_predictor
   stopifnot("x" %in% names(args))
-  p <- combine_prefix(px)
   draws <- list()
+  if (!nrow(ranef)) {
+    return(draws)
+  }
+  p <- combine_prefix(px)
   groups <- unique(ranef$group)
   # assigning S4 objects requires initialisation of list elements
   draws[c("Z", "Zmo", "Zcs", "Zme")] <- list(named_list(groups))
@@ -472,9 +495,11 @@ extract_draws_re <- function(ranef, args, sdata, px = list(),
     r_pars <- paste0("^r_", g, usc(usc(p)), "\\[")
     r <- do.call(as.matrix, c(args, list(pars = r_pars)))
     if (is.null(r)) {
-      stop2("Group-level effects for each level of group ", 
-            "'", g, "' not found. Please set save_ranef = TRUE ",
-            "when calling brm.")
+      stop2(
+        "Group-level effects for each level of group ", 
+        "'", g, "' not found. Please set 'save_ranef' to ", 
+        "TRUE when fitting your model."
+      )
     }
     nlevels <- ngrps(args$x)[[g]]
     old_r <- subset2(args$x$ranef, group = g)
@@ -584,7 +609,7 @@ extract_draws_re <- function(ranef, args, sdata, px = list(),
       Z <- matrix(1, length(gf[[1]]))
       draws[["Zme"]][[g]] <- prepare_Z(Z, gf, max_level, weights)
       draws[["rme"]] <- named_list(new_r_me$coef, list())
-      for (co in names(draws[["r_me"]])) {
+      for (co in names(draws[["rme"]])) {
         take <- which(new_r$coef == co & new_r$type == "me")
         take <- take + nranef * (seq_along(levels) - 1)
         draws[["rme"]][[co]][[g]] <- r[, take, drop = FALSE]
@@ -650,19 +675,22 @@ extract_draws_autocor <- function(fit, subset = NULL,
         stop2("Without a grouping factor, CAR models cannot handle newdata.")
       }
     }
+    draws[["car"]] <- named_list(c("Zcar", "rcar"))
     gcar <- sdata[["Jloc"]]
     Zcar <- matrix(rep(1, length(gcar)))
-    draws[["Zcar"]] <- prepare_Z(Zcar, list(gcar))
+    draws[["car"]][["Zcar"]] <- prepare_Z(Zcar, list(gcar))
     rcar <- do.call(as.matrix, c(args, pars = "^rcar\\["))
     rcar <- rcar[, unique(gcar), drop = FALSE]
-    draws[["rcar"]] <- rcar
+    draws[["car"]][["rcar"]] <- rcar
   }
-  if (is(fit$autocor, "cor_bsts")) {
+  if (is.cor_bsts(fit$autocor)) {
     if (is_newdata) {
       warning2("Local level terms are currently ignored ", 
                "when 'newdata' is specified.")
     } else {
-      draws[["loclev"]] <- do.call(as.matrix, c(args, pars = "^loclev\\["))
+      draws[["loclev"]] <- do.call(
+        as.matrix, c(args, pars = "^loclev\\[")
+      )
     }
   }
   draws
