@@ -86,8 +86,8 @@ test_that("specified priors appear in the Stan code", {
   )
   expect_match2(scode, "target += double_exponential_lpdf(arr | 0, 1)")
   expect_match2(scode, "target += normal_lpdf(bmo | 0, 5)")
-  expect_match2(scode, "target += dirichlet_lpdf(simplex_1 | con_simplex_1)")
-  expect_match2(scode, "prior_simplex_1 = dirichlet_rng(con_simplex_1)")
+  expect_match2(scode, "target += dirichlet_lpdf(simo_1 | con_simo_1)")
+  expect_match2(scode, "prior_simo_1 = dirichlet_rng(con_simo_1)")
   expect_match2(scode, "target += uniform_lpdf(prior_ar | -1,1)")
   
   # test for problem described in #197
@@ -316,7 +316,7 @@ test_that("make_stancode correctly restricts FE parameters", {
   expect_match2(scode, "vector<lower=2>[Kc] b")
   
   scode <- make_stancode(
-    y ~ x, data, prior = set_prior("normal(0, 2)", ub = "4")
+    y ~ x, data, prior = set_prior("normal (0, 2)", ub = "4")
   )
   expect_match2(scode, "vector<upper=4>[Kc] b")
   expect_match2(scode, "- 1 * normal_lcdf(4 | 0, 2)")
@@ -559,28 +559,36 @@ test_that("ordinal disc parameters appear in the Stan code", {
 })
 
 test_that("monotonic effects appear in the Stan code", {
-  prior <- c(prior(normal(0,1), class = b, coef = x1),
-             prior(dirichlet(c(1,0.5,2)), simplex, coef = x1),
-             prior(dirichlet(c(1,0.5,2)), simplex, coef = x2))
+  prior <- c(prior(normal(0,1), class = b, coef = mox1),
+             prior(dirichlet(c(1,0.5,2)), simo, coef = mox11),
+             prior(dirichlet(c(1,0.5,2)), simo, coef = mox21))
   dat <- data.frame(y = rpois(120, 10), x1 = rep(1:4, 30), 
                     x2 = factor(rep(c("a", "b", "c"), 40), ordered = TRUE))
-  scode <- make_stancode(y ~ mo(x1) + mo(x2), dat, prior = prior)
-  expect_match2(scode, "int Xmo[N, Kmo];")
-  expect_match2(scode, "simplex[Jmo[1]] simplex_1;")
-  expect_match2(scode, "(bmo[2]) * mo(simplex_2, Xmo[n, 2]);")
+  scode <- make_stancode(y ~ y*mo(x1)*mo(x2), dat, prior = prior)
+  expect_match2(scode, "int Xmo_3[N];")
+  expect_match2(scode, "simplex[Jmo[1]] simo_1;")
+  expect_match2(scode, "(bmo[2]) * mo(simo_2, Xmo_2[n])")
+  expect_match2(scode, 
+    "(bmo[6]) * mo(simo_7, Xmo_7[n]) * mo(simo_8, Xmo_8[n]) * Cmo_3[n]"
+  )
   expect_match2(scode, "target += normal_lpdf(bmo[1] | 0, 1)")
-  expect_match2(scode, "target += dirichlet_lpdf(simplex_1 | con_simplex_1);")
-  expect_match2(scode, "target += dirichlet_lpdf(simplex_2 | con_simplex_2);")
-  scode <- make_stancode(y ~ mono(x1) + (mono(x1)|x2) + (1|x2), dat)
-  expect_match2(scode, "(bmo[1] + r_1_1[J_1[n]]) * mo(simplex_1, Xmo[n, 1]);")
-  # test that Z_1_1 is (correctly) undefined
-  expect_match2(scode, paste0("  int<lower=1> M_1; \n",
-    "  // data for group-level effects of ID 2"))
-  expect_error(make_stancode(y ~ mono(x1) + (mono(x1+x2)|x2), dat),
-               "Monotonic group-level terms require")
-  expect_error(make_stancode(y ~ mo(x1), dat, 
-                             prior = prior(beta(1, 1), simplex, coef = x1)),
-               "'dirichlet' is the only valid prior for simplex parameters")
+  expect_match2(scode, "target += dirichlet_lpdf(simo_1 | con_simo_1);")
+  expect_match2(scode, "target += dirichlet_lpdf(simo_8 | con_simo_8);")
+  
+  scode <- make_stancode(y ~ mono(x1) + (mono(x1)|x2), dat)
+  expect_match2(scode, "(bmo[1] + r_1_1[J_1[n]]) * mo(simo_1, Xmo_1[n]);")
+  expect_true(!grepl("Z_1_1", scode))
+  
+  expect_error(
+    make_stancode(y ~ mono(x1) + (mono(x2)|x2), dat),
+    "Monotonic group-level terms require"
+  )
+  
+  prior <- prior(beta(1, 1), simo, coef = mox11)
+  expect_error(
+    make_stancode(y ~ mo(x1), dat, prior = prior),
+    "'dirichlet' is the only valid prior for simplex parameters"
+  )
 })
 
 test_that("Stan code for non-linear models is correct", {
@@ -645,7 +653,7 @@ test_that("no loop in trans-par is defined for simple 'identity' models", {
 test_that("known standard errors appear in the Stan code", {
   scode <- make_stancode(time | se(age) ~ sex, data = kidney)
   expect_match2(scode, "target += normal_lpdf(Y | mu, se)")
-  scode <- make_stancode(time | se(age) | weights(age) ~ sex, data = kidney)
+  scode <- make_stancode(time | se(age) + weights(age) ~ sex, data = kidney)
   expect_match2(scode, "lp_pre[n] = normal_lpdf(Y[n] | mu[n], se[n])")
   scode <- make_stancode(time | se(age, sigma = TRUE) ~ sex, data = kidney)
   expect_match2(scode, "target += normal_lpdf(Y[n] | mu[n], sqrt(sigma^2 + se2[n]))")
@@ -908,21 +916,47 @@ test_that("priors on intercepts appear in the Stan code", {
 
 test_that("noise-free terms appear in the Stan code", {
   N <- 30
-  dat <- data.frame(y = rnorm(N), x = rnorm(N), z = rnorm(N),
-                    xsd = abs(rnorm(N, 1)), zsd = abs(rnorm(N, 1)),
-                    ID = rep(1:5, each = N / 5))
-  scode <- make_stancode(y ~ me(x, xsd)*me(z, zsd)*x, data = dat,
-                      prior = prior(normal(0,5)))
+  dat <- data.frame(
+    y = rnorm(N), x = rnorm(N), z = rnorm(N),
+    xsd = abs(rnorm(N, 1)), zsd = abs(rnorm(N, 1)),
+    ID = rep(1:5, each = N / 5)
+  )
+  scode <- make_stancode(
+    y ~ me(x, xsd)*me(z, zsd)*x, data = dat,
+    prior = prior(normal(0,5))
+  )
   expect_match2(scode,
-    "(bme[1]) * Xme_1[n] + (bme[2]) * Xme_2[n] + (bme[3]) * Xme_1[n] .* Xme_2[n]")
+    "(bme[1]) * Xme_1[n] + (bme[2]) * Xme_2[n] + (bme[3]) * Xme_1[n] * Xme_2[n]")
   expect_match2(scode, 
-    "(bme[6]) * Xme_1[n] .* Xme_2[n] .* Cme_3[n]")
+    "(bme[6]) * Xme_1[n] * Xme_2[n] * Cme_3[n]")
   expect_match2(scode, "target += normal_lpdf(Xme_2 | Xn_2, noise_2)")
   expect_match2(scode, "target += normal_lpdf(bme | 0, 5)")
-  scode <- make_stancode(y ~ me(x, xsd)*me(z, zsd) + (me(x, xsd)|ID), data = dat)
+  
+  scode <- make_stancode(
+    y ~ me(x, xsd)*z + (me(x, xsd)*z|ID), data = dat
+  )
   expect_match2(scode, "(bme[1] + r_1_1[J_1[n]]) * Xme_1[n]")
+  expect_match2(scode, "(bme[2] + r_1_2[J_1[n]]) * Xme_1[n] * Cme_1[n]")
+  
   expect_match2(make_stancode(y ~ I(me(x, xsd)^2), data = dat),
-               "(bme[1]) * Xme_1[n]^2")
+               "(bme[1]) * (Xme_1[n]^2)")
+  
+  # test that noise-free variables are unique across model parts
+  scode <- make_stancode(
+    bf(y ~ me(x, xsd)*me(z, zsd)*x, sigma ~ me(x, xsd)), 
+    data = dat, prior = prior(normal(0,5))
+  )
+  expect_match2(scode, "mu[n] = mu[n] + (bme[1]) * Xme_1[n]")
+  expect_match2(scode, "sigma[n] = sigma[n] + (bme_sigma[1]) * Xme_1[n]")
+  
+  scode <- make_stancode(
+    bf(y ~ a * b, a + b ~ me(x, xsd), nl = TRUE), 
+    data = dat, 
+    prior = prior(normal(0,5), nlpar = a) + 
+      prior(normal(0, 5), nlpar = b)
+  )
+  expect_match2(scode, "mu_a[n] = mu_a[n] + (bme_a[1]) * Xme_1[n]")
+  expect_match2(scode, "mu_b[n] = mu_b[n] + (bme_b[1]) * Xme_1[n]")
 })
 
 test_that("Stan code of multi-membership models is correct", {
